//
//  ALPanel.m
//  ALKit
//
//  Created by Aaron Levine on 5/6/13.
//  Copyright (c) 2013 Aaron Levine. All rights reserved.
//

#import "ALPanel.h"

@interface ALPanel ()

- (int **)makeLayoutMatrix;
- (NSArray *)makeViews;
- (NSArray *)makeFrames;
- (ALPanelPosition)panelPositionForViewAtIndex:(int)index;

@end

@implementation ALPanel

- (id)initWithOrigin:(CGPoint)origin numberOfViews:(int)numViews numberOfColumns:(int)numColumns numberOfRows:(int)numRows dataSource:(id)dataSource
{
    _numViews = numViews; _numColumns = numColumns; _numRows = numRows; _dataSource = dataSource;
    
    //Get Width
    float width = 0;
    for (int i = 0; i < numColumns; i++) {
        width += [_dataSource panel:self widthForColumn:i];
    }
    //Get Height
    float height = 0;
    for (int i = 0; i < numRows; i++) {
        height += [_dataSource panel:self heightForRow:i];
    }
    CGRect frame; frame.origin = origin; frame.size.width = width; frame.size.height = height;
    self = [super initWithFrame:frame];
    
    return self;
}

- (void)layoutSubviews
{
    _layoutMatrix = [self makeLayoutMatrix];
    _views = [self makeViews];
    
    for (UIView *view in self.subviews)
        [view removeFromSuperview];
    
    for (UIView *view in _views)
        [self addSubview:view];
    
}

- (int **)makeLayoutMatrix
{
    int **layoutMatrix = malloc(_numColumns * sizeof(int));
    for (int i = 0; i < _numColumns; i++) {
        layoutMatrix[i] = malloc(_numRows * sizeof(int));
    }
    for (int i = 0; i < _numColumns; i++) {
        for (int j = 0; j < _numRows; j++) {
            layoutMatrix[i][j] = [_dataSource panel:self matrixIndexForColumn:i Row:j];
        }
    }
    return layoutMatrix;
}

- (NSArray *)makeViews
{
    NSMutableArray *viewsMutable = [[NSMutableArray alloc] initWithCapacity:_numViews];
    NSArray *frames = [self makeFrames];
    for (int i = 0; i < _numViews; i++) {
        CGRect frame = [[frames objectAtIndex:i] CGRectValue];
        UIView *view = [_dataSource panel:self viewForMatrixIndex:i size:CGSizeMake(frame.size.width - 1, frame.size.height - 1)];
        view.frame = frame;
        
        if ([view isKindOfClass:[ALPanelCell class]]) {
            ALPanelCell *panelCell = (ALPanelCell *)view;
            panelCell.panel = [self panelPositionForViewAtIndex:i];
            view = panelCell;
            
        } else if ([view isKindOfClass:[ALPanelButton class]]) {
            ALPanelButton *panelButton = (ALPanelButton *)view;
            panelButton.panel = [self panelPositionForViewAtIndex:i];
            view = panelButton;
        }
        [viewsMutable insertObject:view atIndex:i];
        //NSLog(@"%i: %@", i, view.debugDescription);
    }
    NSArray *views = [[NSArray alloc] initWithArray:viewsMutable];
    return views;
}

- (NSArray *)makeFrames
{
    NSMutableArray *frames = [[NSMutableArray alloc] init];
    CGRect nilRect = CGRectMake(0, 0, 0, 0);
    for (int i = 0; i < _numViews; i++) {
        [frames addObject:[NSValue valueWithCGRect:nilRect]];
    }
    
    for (int i = 0; i < _numColumns; i++) {
        for (int j = 0; j < _numRows; j++) {
                        
            //Get Previous Widths
            float previousWidths = 0;
            for (int w = i-1; w >= 0; w--) {
                previousWidths += [_dataSource panel:self widthForColumn:w];
            }
            
            //Get Previous Heights
            float previousHeights = 0;
            for (int h = j-1; h >= 0; h--) {
                previousHeights += [_dataSource panel:self heightForRow:h];
            }
            
            //Create This Frame
            float thisX = _origin.x + (previousWidths - i);
            float thisY = _origin.y + (previousHeights - j);
            float thisWidth = [_dataSource panel:self widthForColumn:i] + i;
            float thisHeight = [_dataSource panel:self heightForRow:j] + j;
            CGRect thisFrame = CGRectMake(thisX, thisY, thisWidth, thisHeight);
            
            //Add Frame To Frame With Same Index
            if (!CGRectEqualToRect([[frames objectAtIndex:_layoutMatrix[i][j]] CGRectValue], nilRect)) {
                
                CGRect previousFrames = [[frames objectAtIndex:_layoutMatrix[i][j]] CGRectValue];
                
                CGRect newFrame = CGRectUnion(previousFrames, thisFrame);
                
                [frames replaceObjectAtIndex:_layoutMatrix[i][j] withObject:[NSValue valueWithCGRect:newFrame]];
            }
            else
                [frames replaceObjectAtIndex:_layoutMatrix[i][j] withObject:[NSValue valueWithCGRect:thisFrame]];
        }
    }
    return frames;
}

- (ALPanelPosition)panelPositionForViewAtIndex:(int)index
{
    bool topLeft = NO, topRight = NO, bottomLeft = NO, bottomRight = NO;
    if (_layoutMatrix[0][0] == index)
        topLeft = YES;
    if (_layoutMatrix[_numColumns - 1][0] == index)
        topRight = YES;
    if (_layoutMatrix[0][_numRows - 1] == index)
        bottomLeft = YES;
    if (_layoutMatrix[_numColumns - 1][_numRows - 1] == index)
        bottomRight = YES;
    
    ALPanelPosition panel;
    if (topLeft && topRight && bottomLeft && bottomRight)
        panel = ALPanelPositionUnbounded;
    
    else if (topLeft && bottomLeft)
        panel = ALPanelPositionLeft;
    else if (topLeft && topRight)
        panel = ALPanelPositionTop;
    else if (topRight && bottomRight)
        panel = ALPanelPositionRight;
    else if (bottomLeft && bottomRight)
        panel = ALPanelPositionBottom;
    
    else if (topLeft)
        panel = ALPanelPositionTopLeft;
    else if (topRight)
        panel = ALPanelPositionTopRight;
    else if (bottomLeft)
        panel = ALPanelPositionBottomLeft;
    else if (bottomRight)
        panel = ALPanelPositionBottomRight;
    
    else
        panel = ALPanelPositionCenter;
    
    return panel;
}

@end